@@ -33,6 +33,8 @@ class InvoiceIndex(CustomSearchIndex):
 
     # PK: needed for exclude list_tags
     primary_key = indexes.CharField(model_attr='pk')
+    
+    order = indexes.DateTimeField()
 
     def get_updated_field(self):
         return 'update_dt'
@@ -42,6 +44,9 @@ class InvoiceIndex(CustomSearchIndex):
         if myobj:
             return myobj._meta.verbose_name
         return obj.object_type
+    
+    def prepare_order(self, obj):
+        return obj.create_dt
 
 # Removed from index after search view was updated to perform
 # all searches on the database.
