@@ -10,7 +10,7 @@ from tendenci.core.categories.forms import (CategoryForm, CategoryField, categor
 
 from tendenci.core.categories.models import CategoryItem, Category
 from tendenci.core.files.fields import MultiFileField
-from tendenci.core.files.models import File
+from tendenci.core.files.models import File, FilesCategory
 from tendenci.core.files.utils import get_max_file_upload_size
 from tendenci.core.perms.fields import GroupPermissionField, groups_with_perms, UserPermissionField, MemberPermissionField, group_choices
 from tendenci.core.perms.forms import TendenciBaseForm
@@ -23,6 +23,14 @@ from form_utils.forms import BetterForm
 class FileForm(TendenciBaseForm):
 
     group = forms.ChoiceField(required=True, choices=[])
+    file_cat = forms.ModelChoiceField(label=_("Category"),
+                                      queryset=FilesCategory.objects.filter(parent=None),
+                                      empty_label="-----------",
+                                      required=False)
+    file_sub_cat = forms.ModelChoiceField(label=_("Sub-Category"),
+                                          queryset=FilesCategory.objects.none(),
+                                          empty_label="Please choose a category first",
+                                          required=False)
 
     class Meta:
         model = File
@@ -36,6 +44,8 @@ class FileForm(TendenciBaseForm):
             'user_perms',
             'member_perms',
             'group_perms',
+            'file_cat',
+            'file_sub_cat'
         )
 
         fieldsets = [('', {
@@ -56,7 +66,12 @@ class FileForm(TendenciBaseForm):
                                  ],
                       'classes': ['permissions'],
                       }),
-
+                      ('Category', {
+                        'fields': ['file_cat',
+                                   'file_sub_cat'
+                                   ],
+                        'classes': ['boxy-grey'],
+                      }),
                      ('Administrator Only', {
                       'fields': ['status_detail'],
                       'classes': ['admin-only'],
@@ -66,6 +81,11 @@ class FileForm(TendenciBaseForm):
         super(FileForm, self).__init__(*args, **kwargs)
         default_groups = Group.objects.filter(status=True, status_detail="active")
 
+        if args:
+            post_data = args[0]
+        else:
+            post_data = None
+
         if self.user and not self.user.profile.is_superuser:
             filters = get_query_filters(self.user, 'user_groups.view_group', **{'perms_field': False})
             groups = default_groups.filter(filters).distinct()
@@ -80,6 +100,16 @@ class FileForm(TendenciBaseForm):
 
         self.fields['group'].choices = groups_list
 
+        if self.instance and self.instance.pk:
+            self.fields['file_sub_cat'].queryset = FilesCategory.objects.filter(
+                                                        parent=self.instance.file_cat)
+
+        if post_data:
+            file_cat = post_data.get('file_cat', '0')
+            if file_cat and file_cat != '0' and file_cat != u'': 
+                file_cat = FilesCategory.objects.get(pk=int(file_cat))
+                self.fields['file_sub_cat'].queryset = FilesCategory.objects.filter(parent=file_cat)
+
     def clean_file(self):
         data = self.cleaned_data.get('file')
         max_upload_size = get_max_file_upload_size(file_module=True)
@@ -148,8 +178,14 @@ class MostViewedForm(forms.Form):
 
 class FileSearchForm(forms.Form):
     q = forms.CharField(label=_("Search"), required=False, max_length=200,)
-    category = CategoryField(label=_('Category'), choices=[], required=False)
-    sub_category = CategoryField(label=_('Sub Category'), choices=[], required=False)
+    file_cat = forms.ModelChoiceField(label=_("Category"),
+                                      queryset=FilesCategory.objects.filter(parent=None),
+                                      empty_label="-----------",
+                                      required=False)
+    file_sub_cat = forms.ModelChoiceField(label=_("Sub-Category"),
+                                          queryset=FilesCategory.objects.none(),
+                                          empty_label="Please choose a category first",
+                                          required=False)
     group = forms.ChoiceField(label=_('Group'), choices=[], required=False)
 
     def __init__(self, *args, **kwargs):
@@ -158,6 +194,9 @@ class FileSearchForm(forms.Form):
         else:
             self.user = None
 
+        data = args[0] if args else kwargs['data'] if 'data' in kwargs else None
+        category = data['file_cat'] if data and 'file_cat' in data else None
+
         super(FileSearchForm, self).__init__(*args, **kwargs)
 
         user = self.user or AnonymousUser()
@@ -173,29 +212,27 @@ class FileSearchForm(forms.Form):
         groups_list.insert(0, ['', '------------'])
         self.fields['group'].choices = tuple(groups_list)
 
-        content_type = ContentType.objects.get(app_label='files', model='file')
-        categories = CategoryItem.objects.filter(content_type=content_type,
-                                                 parent__exact=None)
-        categories = list(set([cat.category.name for cat in categories]))
-        categories = [[cat, cat] for cat in categories]
-        categories.insert(0, ['', '------------'])
-        self.fields['category'].choices = tuple(categories)
-
-        # set up the sub category choices
-        sub_categories = CategoryItem.objects.filter(content_type=content_type,
-                                                     category__exact=None)
-        sub_categories = list(set([cat.parent.name for cat in sub_categories]))
-        sub_categories = [[cat, cat] for cat in sub_categories]
-        sub_categories.insert(0, ['', '------------'])
-        self.fields['sub_category'].choices = tuple(sub_categories)
+        # Update categories and subcategories choices
+        main_categories = FilesCategory.objects.filter(parent__isnull=True)
+        self.fields['file_cat'].queryset = main_categories
+        if category:
+            sub_categories = FilesCategory.objects.filter(parent=category)
+            self.fields['file_sub_cat'].empty_label = "-----------"
+            self.fields['file_sub_cat'].queryset = sub_categories
 
 
 class FilewithCategoryForm(TendenciBaseForm):
 
     group = forms.ChoiceField(required=True, choices=[])
 
-    category = CategoryField(required=False, **category_defaults)
-    sub_category = CategoryField(required=False, **sub_category_defaults)
+    file_cat = forms.ModelChoiceField(label=_("Category"),
+                                      queryset=FilesCategory.objects.filter(parent=None),
+                                      empty_label="-----------",
+                                      required=False)
+    file_sub_cat = forms.ModelChoiceField(label=_("Sub-Category"),
+                                          queryset=FilesCategory.objects.none(),
+                                          empty_label="Please choose a category first",
+                                          required=False)
 
     class Meta:
         model = File
@@ -205,8 +242,8 @@ class FilewithCategoryForm(TendenciBaseForm):
             'name',
             'group',
             'tags',
-            'category',
-            'sub_category',
+            'file_cat',
+            'file_sub_cat',
             'allow_anonymous_view',
             'user_perms',
             'member_perms',
@@ -236,36 +273,15 @@ class FilewithCategoryForm(TendenciBaseForm):
 
         self.fields['group'].choices = groups_list
 
-        content_type = ContentType.objects.get(app_label='files', model='file')
-        categories = CategoryItem.objects.filter(content_type=content_type,
-                                                 parent__exact=None)
-        categories = list(set([cat.category.name for cat in categories]))
-        categories = [[cat, cat] for cat in categories]
-        categories.insert(0, [0, '------------'])
-        if post_data:
-            new_category = post_data.get('category','0')
-            if new_category != '0':
-                categories.append([new_category,new_category])
-        self.fields['category'].choices = tuple(categories)
-
-        # set up the sub category choices
-        sub_categories = CategoryItem.objects.filter(content_type=content_type,
-                                                     category__exact=None)
-        sub_categories = list(set([cat.parent.name for cat in sub_categories]))
-        sub_categories = [[cat, cat] for cat in sub_categories]
-        sub_categories.insert(0, [0, '------------'])
-        if post_data:
-            new_sub_category = post_data.get('sub_category','0')
-            if new_sub_category != '0':
-                sub_categories.append([new_sub_category,new_sub_category])
-        self.fields['sub_category'].choices = tuple(sub_categories)
-
         if self.instance and self.instance.pk:
-            category = Category.objects.get_for_object(self.instance, 'category')
-            sub_category = Category.objects.get_for_object(self.instance, 'sub_category')
-            self.fields['category'].initial = category
-            self.fields['sub_category'].initial = sub_category
+            self.fields['file_sub_cat'].queryset = FilesCategory.objects.filter(
+                                                        parent=self.instance.file_cat)
 
+        if post_data:
+            file_cat = post_data.get('file_cat', '0')
+            if file_cat and file_cat != '0' and file_cat != u'':
+                file_cat = FilesCategory.objects.get(pk=int(file_cat))
+                self.fields['file_sub_cat'].queryset = FilesCategory.objects.filter(parent=file_cat)
 
     def clean_file(self):
         data = self.cleaned_data.get('file')
@@ -296,9 +312,9 @@ class FilewithCategoryForm(TendenciBaseForm):
 
         ## update the category of the file
         category_removed = False
-        category = data.get('category')
+        category = file.file_cat.name if file.file_cat else u''
 
-        if category != '0':
+        if category:
             Category.objects.update(file, category, 'category')
         else:  # remove
             category_removed = True
@@ -307,8 +323,8 @@ class FilewithCategoryForm(TendenciBaseForm):
 
         if not category_removed:
             # update the sub category of the article
-            sub_category = data.get('sub_category')
-            if sub_category != '0':
+            sub_category = file.file_sub_cat.name if file.file_sub_cat else u''
+            if sub_category:
                 Category.objects.update(file, sub_category, 'sub_category')
             else:  # remove
                 Category.objects.remove(file, 'sub_category')
@@ -324,8 +340,14 @@ class MultiFileForm(BetterForm):
     group = forms.ChoiceField(required=True, choices=[])
     tags = forms.CharField(required=False)
 
-    category = CategoryField(required=False, **category_defaults)
-    sub_category = CategoryField(required=False, **sub_category_defaults)
+    file_cat = forms.ModelChoiceField(label=_("Category"),
+                                      queryset=FilesCategory.objects.filter(parent=None),
+                                      empty_label="-----------",
+                                      required=False)
+    file_sub_cat = forms.ModelChoiceField(label=_("Sub-Category"),
+                                          queryset=FilesCategory.objects.none(),
+                                          empty_label="Please choose a category first",
+                                          required=False)
 
     allow_anonymous_view = forms.BooleanField(label=_("Public can View"), initial=True, required=False)
 
@@ -341,7 +363,7 @@ class MultiFileForm(BetterForm):
                            'group',
                            )
             }),
-            ('Category', {'fields': ('category', 'sub_category')}),
+            ('Category', {'fields': ('file_cat', 'file_sub_cat')}),
             ('Permissions', {'fields': ('allow_anonymous_view',)}),
             ('Advanced Permissions', {'classes': ('collapse',), 'fields': (
                 'user_perms',
@@ -379,29 +401,12 @@ class MultiFileForm(BetterForm):
 
         self.fields['group'].choices = groups_list
 
-        content_type = ContentType.objects.get(app_label='files', model='file')
-        categories = CategoryItem.objects.filter(content_type=content_type,
-                                                 parent__exact=None)
-        categories = list(set([cat.category.name for cat in categories]))
-        categories = [[cat, cat] for cat in categories]
-        categories.insert(0, [0, '------------'])
-        if post_data:
-            new_category = post_data.get('category','0')
-            if new_category != '0':
-                categories.append([new_category,new_category])
-        self.fields['category'].choices = tuple(categories)
-
         # set up the sub category choices
-        sub_categories = CategoryItem.objects.filter(content_type=content_type,
-                                                     category__exact=None)
-        sub_categories = list(set([cat.parent.name for cat in sub_categories]))
-        sub_categories = [[cat, cat] for cat in sub_categories]
-        sub_categories.insert(0, [0, '------------'])
         if post_data:
-            new_sub_category = post_data.get('sub_category','0')
-            if new_sub_category != '0':
-                sub_categories.append([new_sub_category,new_sub_category])
-        self.fields['sub_category'].choices = tuple(sub_categories)
+            file_cat = post_data.get('file_cat', '0')
+            if file_cat and file_cat != '0' and file_cat != u'':
+                file_cat = FilesCategory.objects.get(pk=int(file_cat))
+                self.fields['file_sub_cat'].queryset = FilesCategory.objects.filter(parent=file_cat)
 
     def clean_files(self):
         files = self.cleaned_data.get('files')
@@ -474,12 +479,19 @@ class MultiFileForm(BetterForm):
         files = data.get('files')
         tags = data.get('tags')
         group = data.get('group')
-        category_from_form = data.get('category')
-        sub_category_from_form = data.get('sub_category')
+        file_cat = data.get('file_cat', None)
+        file_sub_cat = data.get('file_sub_cat', None)
         is_public = data.get('allow_anonymous_view', False)
 
         for new_file in files:
-            file = File(file=new_file, tags=tags, group=group, allow_anonymous_view=is_public)
+            file = File(
+                file=new_file,
+                tags=tags,
+                group=group, 
+                allow_anonymous_view=is_public,
+                file_cat=file_cat,
+                file_sub_cat=file_sub_cat)
+
             file.save()
 
             # update all permissions and save the model
@@ -491,9 +503,9 @@ class MultiFileForm(BetterForm):
 
             ## update the category of the file
             category_removed = False
-            category = category_from_form
+            category = file.file_cat.name if file.file_cat else u''
 
-            if category != '0':
+            if category:
                 Category.objects.update(file, category, 'category')
             else:  # remove
                 category_removed = True
@@ -502,8 +514,8 @@ class MultiFileForm(BetterForm):
 
             if not category_removed:
                 # update the sub category of the file
-                sub_category = sub_category_from_form
-                if sub_category != '0':
+                sub_category = file.file_sub_cat.name if file.file_sub_cat else u''
+                if sub_category:
                     Category.objects.update(file, sub_category, 'sub_category')
                 else:  # remove
                     Category.objects.remove(file, 'sub_category')
@@ -515,29 +527,42 @@ class MultiFileForm(BetterForm):
         return counter
 
 
-class FileCategoryForm(CategoryForm):
+class FileCategoryForm(forms.Form):
     """
     Form dedicated on adding category to files
     """
 
-    category = CategoryField(required=False, **category_defaults)
-    sub_category = CategoryField(required=False, **sub_category_defaults)
+    file_cat = forms.ModelChoiceField(label=_("Category"),
+                                      queryset=FilesCategory.objects.filter(parent=None),
+                                      empty_label="-----------",
+                                      required=False)
+    file_sub_cat = forms.ModelChoiceField(label=_("Sub-Category"),
+                                          queryset=FilesCategory.objects.none(),
+                                          empty_label="Please choose a category first",
+                                          required=False)
 
-    def __init__(self, content_type, *args, **kwargs):
-        super(FileCategoryForm, self).__init__(content_type, *args, **kwargs)
+    def __init__(self, *args, **kwargs):
+        super(FileCategoryForm, self).__init__(*args, **kwargs)
 
-        self.fields['category'].help_text = mark_safe('<a href="#" class="add-category">+</a>')
-        self.fields['sub_category'].help_text = mark_safe('<a href="#" class="add-sub-category">+</a>')
+        if args:
+            post_data = args[0]
+        else:
+            post_data = None
 
-        del self.fields['app_label']
-        del self.fields['model']
-        del self.fields['pk']
+        if post_data:
+            file_cat = post_data.get('file_cat', '0')
+            if file_cat and file_cat != '0' and file_cat != u'':
+                file_cat = FilesCategory.objects.get(pk=int(file_cat))
+                self.fields['file_sub_cat'].queryset = FilesCategory.objects.filter(parent=file_cat)
 
     def update_file_cat_and_sub_cat(self, file):
         data = self.cleaned_data
 
-        category_from_form = data.get('category')
-        sub_category_from_form = data.get('sub_category')
+        file_cat = data.get('file_cat')
+        file_sub_cat = data.get('file_sub_cat')
+
+        file.file_cat = file_cat
+        file.file_sub_cat = file_sub_cat
 
         #setup categories
         category = Category.objects.get_for_object(file, 'category')
@@ -545,9 +570,9 @@ class FileCategoryForm(CategoryForm):
 
         ## update the category of the file
         category_removed = False
-        category = category_from_form
+        category = file.file_cat.name if file.file_cat else u''
 
-        if category != '0':
+        if category:
             Category.objects.update(file, category, 'category')
         else:  # remove
             category_removed = True
@@ -556,8 +581,8 @@ class FileCategoryForm(CategoryForm):
 
         if not category_removed:
             # update the sub category of the file
-            sub_category = sub_category_from_form
-            if sub_category != '0':
+            sub_category = file.file_sub_cat.name if file.file_sub_cat else u''
+            if sub_category:
                 Category.objects.update(file, sub_category, 'sub_category')
             else:  # remove
                 Category.objects.remove(file, 'sub_category')
